package com.ruoyi.receive.kafka.config;

import com.ruoyi.receive.netty.server.KafkaConsumerService;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Collections;
import java.util.Properties;

/**
 * 消费者
 */

@Component
public class Consumer {

    private final KafkaConfig kafkaConfig;

    private final ConsumerCustomConfig consumerCustomConfig;

    private final KafkaConsumerService kafkaConsumerService;

    public Consumer(KafkaConfig kafkaConfig, ConsumerCustomConfig consumerCustomConfig, KafkaConsumerService kafkaConsumerService) {
        this.kafkaConfig = kafkaConfig;
        this.consumerCustomConfig = consumerCustomConfig;
        this.kafkaConsumerService = kafkaConsumerService;
    }

    @Bean
    public KafkaConsumer<String,String> consumerInit(){
        //创建kafka消费者配置
        Properties properties = new Properties();
        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG,kafkaConfig.getHosts() );
        properties.put(ConsumerConfig.GROUP_ID_CONFIG,consumerCustomConfig.getGroup());
        properties.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        properties.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG,StringDeserializer.class.getName());
        //创建kafka消费者
        KafkaConsumer<String,String> consumer= new KafkaConsumer<>(properties);
        //分配特定的分区
        TopicPartition topicPartition = new TopicPartition(consumerCustomConfig.getTopic(), consumerCustomConfig.getPartitions());
        consumer.assign(Collections.singletonList(topicPartition));
        new Thread(()->{

            //消费信息
            while (true){

                ConsumerRecords<String,String> records =
                        consumer.poll(Duration.ofMillis(consumerCustomConfig.getOfMillis()));
                kafkaConsumerService.service(records);

            }

        }).start();
        return consumer;
    }



}
