package com.ruoyi.receive.netty.handler;

import com.alibaba.fastjson.JSON;
import com.ruoyi.receive.netty.config.Config;
import com.ruoyi.receive.kafka.config.KafkaSender;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleStateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import java.net.SocketAddress;

/**
 * netty服务端处理器
 * @Author： JCC
 * @Date： 2023/8/17 18:58
 * @Description：
 */
public class NettyServerHandler extends ChannelInboundHandlerAdapter {

    private static final Logger log = LoggerFactory.getLogger(NettyServerHandler.class);
    int readTime = 0;

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        IdleStateEvent event = (IdleStateEvent) evt;
        String eventType = null;
        switch (event.state()){
            case READER_IDLE:
                eventType = "读空闲";
                readTime++;
                break;
            case WRITER_IDLE:
                eventType = "写空闲";
                break;
            case ALL_IDLE:
                eventType = "读写空闲";
                break;
            default:
                break;
        }
        log.info(ctx.channel().remoteAddress() + "超时事件：" + eventType);
        if (readTime>10){
            log.info("服务端读空闲超过10次，关闭连接，释放更多资源");
            ctx.writeAndFlush("关闭此连接");
            ctx.channel().close();
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        SocketAddress socketAddress = ctx.channel().remoteAddress();
        log.info(socketAddress + " 已连接");

        // 发送数据
        ctx.writeAndFlush("你好客户端"+ Config.DATA_PACK_SEPARATOR);
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        log.info("客户端信息：" + msg);
        KafkaSender.send("test",2,null, JSON.toJSONString(msg));
    }
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.info(ctx.channel().remoteAddress() + " 已断开连接");
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        ctx.flush();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ctx.close();
    }


}
