package com.ruoyi.receive.netty.server;

import com.ruoyi.receive.netty.config.Config;
import com.ruoyi.receive.netty.handler.NettyServerHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.DelimiterBasedFrameDecoder;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;

/**
 * netty服务端
 * @Author： JCC
 * @Date： 2023/8/17 18:55
 * @Description：
 */
public class NettyServer {

    public void startNetty() throws InterruptedException{
        //创建两个线程组
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup(10); //默认cpu核数*2

        try{
            // 服务端启动类
            ServerBootstrap bootstrap = new ServerBootstrap();
            // 传入两个线程组
            bootstrap.group(bossGroup, workerGroup)
                    // 指定Channel 和NIO一样是采用Channel通道的方式通信 所以需要指定服务端通道
                    .channel(NioServerSocketChannel.class)

                    .option(ChannelOption.SO_BACKLOG, 1024)
                    //保持长连接
                    .childOption(ChannelOption.SO_KEEPALIVE,true)

                    //设置数据处理器
                    .childHandler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel channel) throws Exception {
                            //分包器
                            channel.pipeline().addLast(
                                    new DelimiterBasedFrameDecoder(
                                            1024,
                                            Unpooled.copiedBuffer(Config.DATA_PACK_SEPARATOR.getBytes()
                                            )
                                    )
                            );
                            //编码器
                            channel.pipeline().addLast("encoder", new StringEncoder());
                            //解码器
                            channel.pipeline().addLast("decoder", new StringDecoder());
                            // 在管道中 添加数据处理类
                            channel.pipeline().addLast(new NettyServerHandler());
                            //IdleStateHandler的readerIdleTime参数指定超过10秒还没收到客户端的连接，
                            //会触发IdleStateEvent事件并且交给下一个handler处理，下一个handler必须
                            //实现userEventTriggered方法处理对应事件
                            channel.pipeline().addLast("HBeat", new IdleStateHandler(
                                    10,
                                    20, 0));
                        }
                    });
            // 同步等待成功
            ChannelFuture future = bootstrap.bind(7000).sync();
            if (future.isSuccess()) {
                System.out.println("启动 Netty Server 成功");
            }
            ServiceRegistry serviceRegistry = new ServiceRegistry("10.100.27.4:2181");
            if (serviceRegistry != null) {
                serviceRegistry.register("10.100.27.4" + ":" + 7000);
            }
            //等待服务端监听端口关闭 链路关闭后main函数才会结束
            future.channel().closeFuture().sync();
        } finally {
            // 优雅的关闭 释放资源
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }

}
