package com.ruoyi.receive.netty.server;

import com.ruoyi.receive.netty.config.Constants;
import org.apache.zookeeper.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.io.IOException;
import java.util.concurrent.CountDownLatch;

/**
 * zookeeper注册节点
 * @Author： JCC
 * @Date： 2023/8/20 18:44
 * @Description：
 */
public class ServiceRegistry {

    private static final Logger logger = LoggerFactory.getLogger(ServiceRegistry.class);
    private CountDownLatch latch = new CountDownLatch(1);
    private String registryAddress;
    /**
     * 构造函数，传入zookeeper服务器地址
     * @param registryAddress zookeeper服务器地址
     */
    public ServiceRegistry(String registryAddress) {
        this.registryAddress = registryAddress;
    }
    public void register(String data) {
        if (data != null) {
            // 连接zookeeper服务器
            ZooKeeper zk = connectServer();
            if (zk != null) {
                // 创建节点
                createNode(zk, data);
            }
        }
    }
    /**
     * 连接 zookeeper 服务器
     * @return
     */
    private ZooKeeper connectServer() {
        ZooKeeper zk = null;
        try {
            // 创建ZooKeeper对象，连接zookeeper服务器
            zk = new ZooKeeper(registryAddress, Constants.ZK_SESSION_TIMEOUT, new Watcher() {
                @Override
                public void process(WatchedEvent event) {
                    // 当连接成功时，释放CountDownLatch
                    if (event.getState() == Event.KeeperState.SyncConnected) {
                        latch.countDown();
                    }
                }
            });
            latch.await();
        } catch (IOException | InterruptedException e) {
            logger.error("", e);
        }
        return zk;
    }
    /**
     * 创建节点
     * @param zk
     * @param data
     */
    private void createNode(ZooKeeper zk, String data) {
        try {
            // 将服务数据转换为字节数组
            byte[] bytes = data.getBytes();
            // 创建临时顺序节点
            String path = zk.create(Constants.ZK_DATA_PATH, bytes, ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.EPHEMERAL_SEQUENTIAL);
            logger.debug("创建zookeeper节点 ({} =&gt; {})", path, data);
        } catch (KeeperException | InterruptedException e) {
            logger.error("", e);
        }
    }

}
